/*
 * RT-Linux FIFOs
 *
 *  Copyright (C) 1999 FSM Labs (http://www.fsmlabs.com/)
 *  Written by Michael Barabanov <baraban@fsmlabs.com>
 */

#ifndef __RTF__
#define __RTF__

#define RTF_NO 64
#define RTL_MAX_USER_FIFO RTF_NO

#define RTF_MAX_FIFO (RTL_MAX_USER_FIFO + 8)

#ifdef __KERNEL__


extern int rtf_init(void);

extern int rtf_create_handler(unsigned int fifo, int (*handler)(unsigned int fifo));


typedef int (*rtl_fifo_handler_t)(unsigned int fifo);

/* rtf_put and rtf_get handlers */
extern int rtf_setwritehandler (unsigned int fifo, rtl_fifo_handler_t handler, rtl_fifo_handler_t *oldhandler);
extern int rtf_setreadhandler (unsigned int fifo, rtl_fifo_handler_t handler, rtl_fifo_handler_t *oldhandler);



extern int rtf_create(unsigned int fifo, int size);

extern int rtf_destroy(unsigned int fifo);

extern int rtf_resize(unsigned int minor, int size);

extern int rtf_put(unsigned int fifo, void * buf, int count);

extern int rtf_get(unsigned int fifo, void * buf, int count);


#define RTF_MAXMSGLEN 20

typedef struct {
	int len;
	char data[RTF_MAXMSGLEN];
} rtl_msg_t;

/* mailbox interface to FIFOs */
extern int rtf_put_message (unsigned int fifo, void *msgdata, int msglen)
{
	rtl_msg_t msg;
	if ((unsigned) msglen > RTF_MAXMSGLEN) {
		return -EMSGSIZE;
	}
	msg.len = msglen;
	memcpy(&msg.data, msgdata, msglen);
	
	return rtf_put (fifo, &msg, msglen + sizeof (int));
}

extern int rtf_get_message (unsigned int fifo, void *msg, int msglen)
{
	int len;
	int ret = rtf_get (fifo, &len, sizeof(len));
	if (ret < 0) {
		return ret;
	} else if (len > msglen) {
		return -EMSGSIZE;
	}
	ret = rtf_get(fifo, msg, len);
	if (ret < 0) {
		return ret;
	}
	return len;
}


#endif

#endif
